@@ -22,6 +22,7 @@ from __future__ import print_function
 import math
 import os
 
+import gin
 from mesh_tensorflow import ops_with_redefined_builtins as mtf
 from mesh_tensorflow import tpu_variables
 from mesh_tensorflow import utils
@@ -32,6 +33,7 @@ import tensorflow.compat.v1 as tf
 from tensorflow.python.tpu.ops import tpu_ops  # pylint: disable=g-direct-tensorflow-import
 
 
+@gin.configurable
 class SimdMeshImpl(mtf.MeshImpl):
   """Mesh implementation for TPU using SIMD and MPI operations."""
 
@@ -41,8 +43,7 @@ class SimdMeshImpl(mtf.MeshImpl):
                devices=None,
                device_assignment=None,
                logical_to_physical=None,
-               allreduce_in_bfloat16_max_group_size=32,
-              ):
+               allreduce_in_bfloat16_max_group_size=8):
     """Create a SimdMeshImpl.
 
     Args:
