@@ -50,13 +50,24 @@ class MoE1D(transformer.TransformerLayer):
                second_threshold_train=0.2,
                second_threshold_eval=0.2,
                dropout_rate=0.0,
-               activation="relu"):
+               activation="relu",
+               moe_gating="top_2",
+               min_expert_capacity=4,
+               rand_1_policy_train="input_jitter",
+               rand_1_policy_eval="input_jitter",
+               rand_1_dropout=0.1,
+               rand_1_temperature=1.0,
+               rand_1_jitter=1e-2,
+               switch_top_k=4,
+               output_dim=None,
+               use_experts_attention=False):
     self._hparams = HParams(
-        moe_gating="top_2",
+        moe_gating=moe_gating,
         moe_num_experts=num_experts,
         moe_loss_coef=loss_coef,
         moe_hidden_size=hidden_size,
         moe_group_size=group_size,
+        moe_min_expert_capacity=min_expert_capacity,
         moe_capacity_factor_train=capacity_factor_train,
         moe_capacity_factor_eval=capacity_factor_eval,
         moe_use_second_place_loss=use_second_place_loss,
@@ -64,7 +75,15 @@ class MoE1D(transformer.TransformerLayer):
         moe_second_policy_eval=second_policy_eval,
         moe_second_threshold_train=second_threshold_train,
         moe_second_threshold_eval=second_threshold_eval,
-        moe_dropout_rate=dropout_rate)
+        moe_dropout_rate=dropout_rate,
+        moe_rand_1_policy_train=rand_1_policy_train,
+        moe_rand_1_policy_eval=rand_1_policy_eval,
+        moe_rand_1_dropout=rand_1_dropout,
+        moe_rand_1_temperature=rand_1_temperature,
+        moe_rand_1_jitter=rand_1_jitter,
+        moe_output_dim=output_dim,
+        moe_switch_top_k=switch_top_k,
+        moe_use_experts_attention=use_experts_attention)
     self._activation = activation
 
   def call(self, context, x, losses=None):
@@ -79,20 +98,31 @@ class MoE1D(transformer.TransformerLayer):
           x_shape.dims[:-1] + [mtf.Dimension("length", 1)]
           + x_shape.dims[-1:])
       x = mtf.reshape(x, shape_with_length)
+
+    # Extract the MoE output dimension
+    if self._hparams.moe_output_dim is not None:
+      output_dim = self._hparams.moe_output_dim
+    else:
+      output_dim = context.model.model_dim
     y, loss = transformer_moe_layer_v1(
         x,
-        context.model.model_dim,
+        output_dim,
         self._hparams,
         context.train,
         context.variable_dtype,
         layout=context.model.layout,
         mesh_shape=context.model.mesh_shape,
         nonpadding=context.nonpadding,
-        activation=self._activation)
+        activation=self._activation,
+        num_microbatches=context.num_microbatches)
     if context.losses is not None:
       context.losses.append(loss)
     if not has_length_dim:
-      y = mtf.reshape(y, x_shape)
+      if self._hparams.moe_use_experts_attention:
+        y_reshape = [mtf.reshape(y_out, x_shape) for y_out in y]
+        y = y_reshape
+      else:
+        y = mtf.reshape(y, x_shape)
     return y
 
 
@@ -147,7 +177,8 @@ class MoE2D(transformer.TransformerLayer):
         context.variable_dtype,
         layout=context.model.layout,
         mesh_shape=context.model.mesh_shape,
-        nonpadding=context.nonpadding)
+        nonpadding=context.nonpadding,
+        num_microbatches=context.num_microbatches)
     if context.losses is not None:
       context.losses.append(loss)
     if not has_length_dim:
@@ -157,7 +188,8 @@ class MoE2D(transformer.TransformerLayer):
 
 def transformer_moe_layer_v1(
     inputs, output_dim, hparams, train, variable_dtype,
-    layout=None, mesh_shape=None, nonpadding=None, activation=mtf.relu):
+    layout=None, mesh_shape=None, nonpadding=None, activation=mtf.relu,
+    num_microbatches=None):
   """Local mixture of experts that works well on TPU.
 
   Adapted from the paper https://arxiv.org/abs/1701.06538
@@ -231,6 +263,7 @@ def transformer_moe_layer_v1(
       and the same dtype as inputs, consisting of ones(nonpadding)
       and zeros(padding).
     activation: a function.
+    num_microbatches: number of microbatches.
 
   Returns:
     outputs: a Tensor with shape [batch_dim(s), length_dim, output_dim]
@@ -333,8 +366,9 @@ def transformer_moe_layer_v1(
   expert_capacity = min(
       group_size_dim.size,
       int((group_size_dim.size * capacity_factor) / experts_dim.size))
+  expert_capacity = max(expert_capacity, hparams.moe_min_expert_capacity)
+  tf.logging.info("expert_capacity: %d" % expert_capacity)
   expert_capacity_dim = mtf.Dimension("expert_capacity", expert_capacity)
-
   experts_dim_unsplit = mtf.Dimension("expert_unsplit", experts_dim.size)
   batch_dim_unsplit = mtf.Dimension("batch_unsplit", num_groups_dim.size)
   if nonpadding is not None:
@@ -353,7 +387,30 @@ def transformer_moe_layer_v1(
         hparams=hparams,
         train=train,
         variable_dtype=variable_dtype,
-        importance=nonpadding)
+        importance=nonpadding,
+        num_microbatches=num_microbatches)
+  elif hparams.moe_gating == "rand_1":
+    dispatch_tensor, combine_tensor, loss = _rand_1_gating(
+        inputs=inputs,
+        outer_expert_dims=None,
+        experts_dim=experts_dim_unsplit,
+        expert_capacity_dim=expert_capacity_dim,
+        hparams=hparams,
+        train=train,
+        variable_dtype=variable_dtype,
+        importance=nonpadding,
+        num_microbatches=num_microbatches)
+  elif hparams.moe_gating == "switch":
+    dispatch_tensor, combine_tensor, loss = _switch_gating(
+        inputs=inputs,
+        outer_expert_dims=None,
+        experts_dim=experts_dim_unsplit,
+        expert_capacity_dim=expert_capacity_dim,
+        hparams=hparams,
+        train=train,
+        variable_dtype=variable_dtype,
+        importance=nonpadding,
+        num_microbatches=num_microbatches)
   else:
     raise ValueError("unknown hparams.moe_gating=%s" % hparams.moe_gating)
 
@@ -363,6 +420,18 @@ def transformer_moe_layer_v1(
                                  num_groups_dim, expert_capacity_dim, input_dim
                              ]))
 
+  # Extra reshape reduces communication cost for model-parallel versions.
+  # For model-parallel versions, this reshape causes an mtf.slice and for non-
+  # model-parallel versions, this has no effect.
+  d_model_split_dim = mtf.Dimension("d_model_split", input_dim.size)
+  expert_inputs = mtf.reshape(
+      expert_inputs,
+      mtf.Shape([
+          outer_batch_dim, experts_dim, batch_dim_unsplit, expert_capacity_dim,
+          d_model_split_dim
+      ]))
+
+  # Split over batch -> split over experts
   expert_inputs = mtf.reshape(
       expert_inputs,
       mtf.Shape([
@@ -382,32 +451,56 @@ def transformer_moe_layer_v1(
   if train and hparams.moe_dropout_rate != 0.0:
     h = mtf.dropout(h, 1.0 - hparams.moe_dropout_rate)
 
-  expert_output = mtf.layers.dense(
-      h, output_dim, expert_dims=[experts_dim], use_bias=False,
-      reduced_dims=h.shape.dims[-1:], variable_dtype=variable_dtype,
-      name="wo")
+  def _compute_output(hidden, layer_name):
+    """Compute the output of the attention layer from the hidden vector."""
+    expert_output = mtf.layers.dense(
+        hidden, output_dim, expert_dims=[experts_dim], use_bias=False,
+        reduced_dims=hidden.shape.dims[-1:], variable_dtype=variable_dtype,
+        name=layer_name)
 
-  expert_output = mtf.reshape(
-      expert_output,
-      mtf.Shape([
-          outer_batch_dim,
-          experts_dim_unsplit,
-          num_groups_dim,
-          expert_capacity_dim,
-          output_dim,
-      ]))
+    # Extra reshape reduces communication cost for model-parallel versions.
+    # For model-parallel versions, this reshape causes an mtf.slice and for non-
+    # model-parallel versions, this has no effect.
+    expert_output = mtf.reshape(
+        expert_output,
+        mtf.Shape([
+            outer_batch_dim, experts_dim_unsplit, num_groups_dim,
+            expert_capacity_dim, d_model_split_dim
+        ]))
 
-  moe_output_dims = moe_input_dims[:-1] + [output_dim]
-  output = mtf.einsum([expert_output, combine_tensor],
-                      mtf.Shape(moe_output_dims))
-  output = mtf.reshape(output, batch_and_length_dims + [output_dim])
+    # Split over experts -> split over batch
+    expert_output = mtf.reshape(
+        expert_output,
+        mtf.Shape([
+            outer_batch_dim,
+            experts_dim_unsplit,
+            num_groups_dim,
+            expert_capacity_dim,
+            output_dim,
+        ]))
+    moe_output_dims = moe_input_dims[:-1] + [output_dim]
+    output = mtf.einsum([expert_output, combine_tensor],
+                        mtf.Shape(moe_output_dims))
+    output = mtf.reshape(output, batch_and_length_dims + [output_dim])
+    return output
 
-  return output, loss * hparams.moe_loss_coef
+  if hparams.moe_use_experts_attention:
+    # We share k_h and v_h with no degradation in performance
+    q_h, k_h = h, h
+    outputs = []
+    q = _compute_output(q_h, layer_name="q_wo")
+    k = _compute_output(k_h, layer_name="k_wo")
+    outputs.append(q)
+    outputs.append(k)
+    return outputs, loss * hparams.moe_loss_coef
+  else:
+    output = _compute_output(h, layer_name="wo")
+    return output, loss * hparams.moe_loss_coef
 
 
 def transformer_moe_layer_v2(
     inputs, output_dim, hparams, train, variable_dtype,
-    layout=None, mesh_shape=None, nonpadding=None):
+    layout=None, mesh_shape=None, nonpadding=None, num_microbatches=None):
   """2-level mixture of experts.
 
   Adapted from the paper https://arxiv.org/abs/1701.06538
@@ -506,6 +599,7 @@ def transformer_moe_layer_v2(
     nonpadding: an optional mtf.Tensor with shape [a, b, l]
       and the same dtype as inputs, consisting of ones(nonpadding)
       and zeros(padding).
+    num_microbatches: number of microbatches.
 
   Returns:
     outputs: a Tensor with shape [a, b, l, n]
@@ -548,7 +642,7 @@ def transformer_moe_layer_v2(
   else:
     capacity_factor = hparams.moe_capacity_factor_eval
   expert_capacity = min(s.size, int((s.size * capacity_factor) / x.size))
-  expert_capacity = max(expert_capacity, 4)
+  expert_capacity = max(expert_capacity, hparams.moe_min_expert_capacity)
   c = mtf.Dimension("expert_capacity_x", expert_capacity)
 
   # We "cheat" here and look at the mesh shape and layout. This is to ensure
@@ -565,7 +659,7 @@ def transformer_moe_layer_v2(
   expert_capacity = min(
       t.size,
       int((t.size * hparams.moe_capacity_factor_second_level) / y.size))
-  expert_capacity = max(expert_capacity, 4)
+  expert_capacity = max(expert_capacity, hparams.moe_min_expert_capacity)
   d = mtf.Dimension("expert_capacity_y", expert_capacity)
 
   # First level of expert routing
@@ -587,7 +681,8 @@ def transformer_moe_layer_v2(
         train=train,
         variable_dtype=variable_dtype,
         name="outer_gating",
-        importance=nonpadding)
+        importance=nonpadding,
+        num_microbatches=num_microbatches)
   else:
     raise ValueError("unknown hparams.moe_gating=%s" % hparams.moe_gating)
 
@@ -627,7 +722,8 @@ def transformer_moe_layer_v2(
         train=train,
         variable_dtype=variable_dtype,
         importance=importance,
-        name="inner_gating")
+        name="inner_gating",
+        num_microbatches=num_microbatches)
   else:
     raise ValueError("unknown hparams.moe_gating=%s" % hparams.moe_gating)
 
@@ -678,9 +774,262 @@ def transformer_moe_layer_v2(
   return output, (loss_outer + loss_inner) * hparams.moe_loss_coef
 
 
+def _switch_gating(inputs,
+                   outer_expert_dims,
+                   experts_dim,
+                   expert_capacity_dim,
+                   hparams,
+                   train,
+                   variable_dtype,
+                   importance=None,
+                   name="switch_gating",
+                   num_microbatches=None):
+  """Compute a switch top-1 gating with no-token-left behind behavior."""
+  # SELECT EXPERT
+  if train:
+    policy = hparams.moe_rand_1_policy_train
+  else:
+    policy = hparams.moe_rand_1_policy_eval
+
+  # Input perturbations
+  if train and policy == "input_jitter":
+    inputs = mtf.layers.multiplicative_jitter(inputs, hparams.moe_rand_1_jitter)
+
+  gate_logits = mtf.layers.dense(
+      inputs,
+      experts_dim,
+      use_bias=False,
+      expert_dims=outer_expert_dims,
+      variable_dtype=variable_dtype,
+      name=name)
+  raw_gates = mtf.softmax(gate_logits, reduced_dim=experts_dim)
+
+  # The internals of this function run in float32.
+  #   bfloat16 seems to reduce quality.
+  raw_gates = mtf.to_float(raw_gates)
+
+  # Top-k operation
+  k_dim = mtf.Dimension("k", hparams.moe_switch_top_k)
+  expert_gate, expert_index = mtf.top_k(
+      raw_gates, reduced_dim=experts_dim, k_dim=k_dim)
+  expert_mask = mtf.one_hot(expert_index, experts_dim)
+
+  # LOAD BALANCING LOSS
+  outer_batch_dim = inputs.shape[0]
+  batch_dim = inputs.shape[1]
+  group_size_dim = inputs.shape[-2]
+  density_1 = mtf.reduce_mean(expert_mask, reduced_dim=group_size_dim)
+  density_1_proxy = mtf.reduce_mean(raw_gates, reduced_dim=group_size_dim)
+  if importance is not None:
+    expert_mask *= mtf.cast(mtf.equal(importance, 1.0), dtype=raw_gates.dtype)
+    expert_gate *= mtf.cast(mtf.equal(importance, 1.0), dtype=raw_gates.dtype)
+    density_1_proxy *= mtf.cast(
+        mtf.equal(importance, 1.0), dtype=raw_gates.dtype)
+  loss = (
+      mtf.reduce_mean(density_1_proxy * density_1) *
+      float(experts_dim.size * experts_dim.size))
+  if num_microbatches and num_microbatches > 1:
+    tf.logging.info("Dividing load-balance loss by num_microbatches={}".format(
+        num_microbatches))
+    loss /= num_microbatches
+
+  # Logging
+  if train:
+    entropy = mtf.reduce_sum(
+        -raw_gates * mtf.log(raw_gates + 1e-9), reduced_dim=experts_dim)
+    batch_entropy = mtf.reduce_mean(entropy)
+    mtf.scalar_summary(name + "/entropy", batch_entropy)
+
+    mask_count_experts = mtf.reduce_sum(expert_mask, output_shape=[experts_dim])
+    total_routed = mtf.reduce_sum(mask_count_experts)
+    expert_fraction = mtf.to_float(mask_count_experts / total_routed)
+    split_fractions = mtf.split(
+        expert_fraction,
+        split_dim=experts_dim,
+        num_or_size_splits=experts_dim.size)
+    for fraction in split_fractions:
+      mtf.scalar_summary("experts/" + fraction.name.replace(":", "/"),
+                         mtf.reduce_mean(fraction))
+    mtf.scalar_summary("aux_loss", mtf.reduce_mean(loss))
+
+  # COMPUTE ASSIGNMENT TO EXPERT
+  # Iteratively route tokens (no-token-left-behind). The idea is to route as
+  # many tokens as possible to top-i before then trying top-(i+1).
+  top_k_masks = mtf.split(
+      expert_mask, split_dim=k_dim, num_or_size_splits=k_dim.size)
+  top_k_gates = mtf.split(
+      expert_gate, split_dim=k_dim, num_or_size_splits=k_dim.size)
+  top_k_indices = mtf.split(
+      expert_index, split_dim=k_dim, num_or_size_splits=k_dim.size)
+
+  # Tensors cumulative values over the iterative process.
+  combine_tensor = mtf.constant(
+      inputs.mesh,
+      value=0,
+      shape=[outer_batch_dim, batch_dim, experts_dim, expert_capacity_dim])
+  cum_tokens = mtf.constant(
+      inputs.mesh, value=0, shape=[outer_batch_dim, batch_dim, experts_dim])
+  tokens_left_to_route = mtf.constant(
+      inputs.mesh, value=1., shape=[outer_batch_dim, batch_dim, group_size_dim])
+
+  expert_capacity_float = float(expert_capacity_dim.size)
+  for (top_i_mask, top_i_gate, top_i_index) in zip(top_k_masks, top_k_gates,
+                                                   top_k_indices):
+    top_i_mask = mtf.reshape(
+        top_i_mask,
+        new_shape=[outer_batch_dim, batch_dim, group_size_dim, experts_dim])
+    # Operate only on the unrouted tokens.
+    top_i_mask *= tokens_left_to_route
+
+    # Record cumulative number of tokens to each expert across iterations.
+    cumulative_tokens_in_expert = cum_tokens + mtf.cumsum(
+        top_i_mask, group_size_dim)
+
+    expert_overflow = mtf.to_float(
+        mtf.less_equal(cumulative_tokens_in_expert, expert_capacity_float))
+    output_i_tokens = top_i_mask * expert_overflow
+
+    # Update the cumulative tokens routed to each expert.
+    cum_tokens += mtf.reduce_sum(output_i_tokens, reduced_dim=group_size_dim)
+    tokens_left_to_route -= (
+        mtf.reduce_sum(output_i_tokens, reduced_dim=experts_dim))
+
+    # Combine-tensor for this iteration
+    output_i_tokens_flat = mtf.reduce_sum(
+        output_i_tokens, reduced_dim=experts_dim)
+    position_in_expert = cumulative_tokens_in_expert - 1
+    top_i_combine_tensor = (
+        top_i_gate * output_i_tokens_flat *
+        mtf.one_hot(top_i_index, experts_dim) *
+        mtf.one_hot(mtf.to_int32(position_in_expert), expert_capacity_dim))
+    combine_tensor += top_i_combine_tensor
+
+  # Match the inputs dtype.
+  combine_tensor = mtf.cast(combine_tensor, inputs.dtype)
+  loss = mtf.cast(loss, inputs.dtype)
+  dispatch_tensor = mtf.cast(
+      mtf.cast(combine_tensor, tf.bool), combine_tensor.dtype)
+
+  return dispatch_tensor, combine_tensor, loss
+
+
+def _rand_1_gating(
+    inputs, outer_expert_dims, experts_dim, expert_capacity_dim,
+    hparams, train, variable_dtype, importance=None, name="rand_1_gating",
+    num_microbatches=None):
+  """Compute a random top-1 gating."""
+  # SELECT EXPERT
+  if train:
+    policy = hparams.moe_rand_1_policy_train
+  else:
+    policy = hparams.moe_rand_1_policy_eval
+
+  # The internals of this function run in float32.
+  #   bfloat16 seems to reduce quality.
+  gate_inputs = mtf.to_float(inputs)
+
+  # Input perturbations
+  if train and policy == "input_dropout":
+    gate_inputs = mtf.dropout(gate_inputs, 1.0 - hparams.moe_rand_1_dropout)
+  elif train and policy == "input_jitter":
+    gate_inputs = mtf.layers.multiplicative_jitter(gate_inputs,
+                                                   hparams.moe_rand_1_jitter)
+
+  gate_logits = mtf.layers.dense(
+      gate_inputs,
+      experts_dim,
+      use_bias=False,
+      expert_dims=outer_expert_dims,
+      variable_dtype=variable_dtype,
+      name=name)
+  raw_gates = mtf.softmax(gate_logits, reduced_dim=experts_dim)
+
+  if policy == "argmax" or policy == "input_dropout" or policy == "input_jitter":
+    expert_gate, expert_index = mtf.top_1(raw_gates, reduced_dim=experts_dim)
+  elif policy == "sample":
+    expert_index = mtf.sample_with_temperature(
+        gate_logits, experts_dim, temperature=hparams.moe_rand_1_temperature)
+    expert_gate = mtf.gather(raw_gates, expert_index, dim=experts_dim)
+  else:
+    raise ValueError("Unknown rand_1 policy %s" % policy)
+
+  expert_mask = mtf.one_hot(expert_index, experts_dim, dtype=raw_gates.dtype)
+
+  # LOAD BALANCING LOSS
+  # TODO(liamfedus): Check entropy loss.
+  group_size_dim = inputs.shape[-2]
+  density_1 = mtf.reduce_mean(expert_mask, reduced_dim=group_size_dim)
+  density_1_proxy = mtf.reduce_mean(raw_gates, reduced_dim=group_size_dim)
+  if importance is not None:
+    expert_mask *= mtf.cast(mtf.equal(importance, 1.0), dtype=raw_gates.dtype)
+    expert_gate *= mtf.cast(mtf.equal(importance, 1.0), dtype=raw_gates.dtype)
+    density_1_proxy *= mtf.cast(
+        mtf.equal(importance, 1.0), dtype=raw_gates.dtype)
+  loss = (
+      mtf.reduce_mean(density_1_proxy * density_1) *
+      float(experts_dim.size * experts_dim.size))
+  if num_microbatches and num_microbatches > 1:
+    tf.logging.info("Dividing load-balance loss by num_microbatches={}".format(
+        num_microbatches))
+    loss /= num_microbatches
+
+  # Logging
+  if train:
+    entropy = mtf.reduce_sum(-raw_gates * mtf.log(raw_gates + 1e-9),
+                             reduced_dim=experts_dim)
+    batch_entropy = mtf.reduce_mean(entropy)
+    mtf.scalar_summary(name + "/entropy", batch_entropy)
+
+    mask_count_experts = mtf.reduce_sum(expert_mask, output_shape=[experts_dim])
+    total_routed = mtf.reduce_sum(mask_count_experts)
+    expert_fraction = mtf.to_float(mask_count_experts / total_routed)
+    split_fractions = mtf.split(
+        expert_fraction,
+        split_dim=experts_dim,
+        num_or_size_splits=experts_dim.size)
+    for fraction in split_fractions:
+      mtf.scalar_summary("experts/" + fraction.name.replace(":", "/"),
+                         mtf.reduce_mean(fraction))
+    mtf.scalar_summary("aux_loss", mtf.reduce_mean(loss))
+
+  # COMPUTE ASSIGNMENT TO EXPERT
+  # Experts have a limited capacity, ensure we do not exceed it. Construct
+  # the batch indices, to each expert, with position_in_expert
+  position_in_expert = mtf.cumsum(
+      expert_mask, group_size_dim, exclusive=True) * expert_mask
+  position_in_expert = mtf.cast(position_in_expert, dtype=raw_gates.dtype)
+  # Keep only tokens that fit within expert_capacity.
+  expert_capacity_float = float(expert_capacity_dim.size)
+  expert_mask *= mtf.cast(
+      mtf.less(position_in_expert, expert_capacity_float),
+      dtype=raw_gates.dtype)
+  expert_mask_flat = mtf.reduce_sum(expert_mask, reduced_dim=experts_dim)
+
+  # Mask out the experts that have overflowed expert capacity. Sparsify the
+  # expert_gate.
+  expert_gate *= expert_mask_flat
+
+  combine_tensor = (
+      expert_gate * expert_mask_flat *
+      mtf.one_hot(expert_index, experts_dim, dtype=raw_gates.dtype) *
+      mtf.one_hot(
+          mtf.to_int32(position_in_expert),
+          expert_capacity_dim,
+          dtype=raw_gates.dtype))
+
+  # Match the inputs dtype.
+  combine_tensor = mtf.cast(combine_tensor, inputs.dtype)
+  loss = mtf.cast(loss, inputs.dtype)
+  dispatch_tensor = mtf.cast(
+      mtf.cast(combine_tensor, tf.bool), combine_tensor.dtype)
+
+  return dispatch_tensor, combine_tensor, loss
+
+
 def _top_2_gating(
     inputs, outer_expert_dims, experts_dim, expert_capacity_dim,
-    hparams, train, variable_dtype, importance=None, name="top_2_gating"):
+    hparams, train, variable_dtype, importance=None, name="top_2_gating",
+    num_microbatches=None):
   """Compute gating for mixture-of-experts in TensorFlow.
 
   Note: until the algorithm and inferface solidify, we pass in a hyperparameters
@@ -733,6 +1082,7 @@ def _top_2_gating(
     variable_dtype: a mtf.VariableDType
     importance: an optional tensor with shape [<batch_dims>, group_size_dim]
     name: an optional string
+    num_microbatches: number of microbatches.
 
   Returns:
     dispatch_tensor: a Tensor with shape
@@ -746,17 +1096,17 @@ def _top_2_gating(
   """
   group_size_dim, unused_input_dim = inputs.shape.dims[-2:]
 
+  # The internals of this function run in float32.
+  # bfloat16 seems to reduce quality.
+  gate_inputs = mtf.to_float(inputs)
+
   raw_gates = mtf.layers.dense(
-      inputs, experts_dim, use_bias=False,
+      gate_inputs, experts_dim, use_bias=False,
       expert_dims=outer_expert_dims,
       variable_dtype=variable_dtype,
       name=name)
   raw_gates = mtf.softmax(raw_gates, experts_dim)
 
-  # The internals of this function run in float32.
-  #   bfloat16 seems to reduce quality.
-  raw_gates = mtf.to_float(raw_gates)
-
   expert_capacity_f = float(expert_capacity_dim.size)
 
   # FIND TOP 2 EXPERTS PER POSITON
@@ -803,6 +1153,10 @@ def _top_2_gating(
     loss_2 = (mtf.reduce_mean(density_2_proxy * density_2)
               * float(experts_dim.size * experts_dim.size))
     loss += loss_2 * 0.5
+  if num_microbatches and num_microbatches > 1:
+    tf.logging.info("Dividing load-balance loss by num_microbatches={}".format(
+        num_microbatches))
+    loss /= num_microbatches
 
   # Depending on the policy in the hparams, we may drop out some of the
   # second-place experts.
